@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-__version__ = '1.0.0'
+__version__ = '1.2.0'
 
 import argparse
 import gzip
@@ -10,6 +10,7 @@ from pathlib import Path
 import sys
 import tempfile
 import datetime
+from uuid import uuid4
 
 from jsonschema import Draft4Validator, FormatChecker
 from decimal import Decimal
@@ -42,14 +43,14 @@ def add_metadata_values_to_record(record_message, schema_message, timestamp):
     '''Populate metadata _sdc columns from incoming record message
     The location of the required attributes are fixed in the stream
     '''
-    now = datetime.datetime.fromtimestamp(timestamp, tz=datetime.timezone.utc).replace(tzinfo=None).isoformat()
+    utcnow = timestamp.astimezone(datetime.timezone.utc).replace(tzinfo=None).isoformat()
     record_message['record'].update(
-        _sdc_batched_at=now,
+        _sdc_batched_at=utcnow,
         _sdc_deleted_at=record_message.get('record', {}).get('_sdc_deleted_at'),
         _sdc_extracted_at=record_message.get('time_extracted'),
         _sdc_primary_key=schema_message.get('key_properties'),
-        _sdc_received_at=now,
-        _sdc_sequence=int(timestamp * 1e3),
+        _sdc_received_at=utcnow,
+        _sdc_sequence=int(timestamp.timestamp() * 1e3),
         _sdc_table_version=record_message.get('version'))
 
     return record_message['record']
@@ -93,25 +94,20 @@ def float_to_decimal(value):
     return value
 
 
-def get_target_key(message, naming_convention=None, timestamp=None, prefix=None, timezone=None):
-    '''Creates and returns an S3 key for the message'''
-    if not naming_convention:
-        naming_convention = '{stream}-{timestamp}.json'
-
-    # replace simple tokens
-    key = naming_convention.format(
-        stream=message['stream'],
-        timestamp=timestamp if timestamp is not None else datetime.datetime.now(timezone).strftime('%Y%m%dT%H%M%S'),
-        date=datetime.datetime.now(timezone).strftime('%Y%m%d'),
-        time=datetime.datetime.now(timezone).strftime('%H%M%S'))
+def get_target_key(stream, config, timestamp=None, prefix=None):
+    '''Creates and returns an S3 key for the stream'''
 
     # NOTE: Replace dynamic tokens
-    # TODO: replace dynamic tokens such as {date(<format>)} with the date formatted as requested in <format>
+    key = config.get('naming_convention').format(
+        stream=stream,
+        timestamp=timestamp,
+        date=timestamp,
+        uuid=uuid4())
 
     return str(Path(key).parent / f'{prefix}{Path(key).name}') if prefix else key
 
 
-def save_file(file_data, open_func):
+def save_jsonl_file(file_data, open_func):
     if any(file_data['file_data']):
         with open_func(file_data['file_name'], 'at', encoding='utf-8') as output_file:
             output_file.writelines(file_data['file_data'])
@@ -120,50 +116,14 @@ def save_file(file_data, open_func):
         LOGGER.debug("'{}' file saved using open_func '{}'".format(file_data['file_name'], open_func.__name__))
 
 
-def upload_files(file_data, config):
-    s3_client = s3.create_client(config)
-    for stream, file_info in file_data.items():
-        if file_info['file_name'].exists():
-            s3.upload_file(
-                str(file_info['file_name']),
-                s3_client,
-                config.get('s3_bucket'),
-                file_info['target_key'],
-                encryption_type=config.get('encryption_type'),
-                encryption_key=config.get('encryption_key'))
-            LOGGER.debug("{} file {} uploaded to {}".format(stream, file_info['target_key'], config.get('s3_bucket')))
-
-            # NOTE: Remove the local file(s)
-            file_info['file_name'].unlink()
-
-
-def persist_lines(messages, config):
+def persist_lines(messages, config, save_records=save_jsonl_file):
     state = None
     schemas = {}
     key_properties = {}
     validators = {}
 
-    naming_convention_default = '{stream}-{timestamp}.json'
-    naming_convention = config.get('naming_convention') or naming_convention_default
-    open_func = open
     timezone = datetime.timezone(datetime.timedelta(hours=config.get('timezone_offset'))) if config.get('timezone_offset') is not None else None
 
-    if f"{config.get('compression')}".lower() == 'gzip':
-        open_func = gzip.open
-        naming_convention_default = f"{naming_convention_default}.gz"
-        naming_convention = f"{naming_convention}.gz"
-
-    elif f"{config.get('compression')}".lower() == 'lzma':
-        open_func = lzma.open
-        naming_convention_default = f"{naming_convention_default}.xz"
-        naming_convention = f"{naming_convention}.xz"
-
-    elif f"{config.get('compression')}".lower() not in {'', 'none'}:
-        raise NotImplementedError(
-            "Compression type '{}' is not supported. "
-            "Expected: 'none', 'gzip', or 'lzma'"
-            .format(f"{config.get('compression')}".lower()))
-
     # NOTE: Use the system specific temp directory if no custom temp_dir provided
     temp_dir = Path(config.get('temp_dir', tempfile.gettempdir())).expanduser()
 
@@ -172,7 +132,6 @@ def persist_lines(messages, config):
 
     file_data = {}
     now = datetime.datetime.now(timezone)
-    now_formatted = now.strftime('%Y%m%dT%H%M%S')
 
     for message in messages:
         try:
@@ -188,33 +147,20 @@ def persist_lines(messages, config):
             if stream not in schemas:
                 raise Exception('A record for stream {} was encountered before a corresponding schema'.format(stream))
 
-            # NOTE: Validate record
             record_to_load = o['record']
-            try:
-                validators[stream].validate(float_to_decimal(record_to_load))
-            except Exception as ex:
-                # NOTE: let anything but 'InvalidOperation' raised Exception slip by
-                # And actual references of the validator logic can be find
-                # at https://github.com/Julian/jsonschema/blob/main/jsonschema/_validators.py
-                # logic covered in the 'jsonschema' package
-                if type(ex).__name__ == "InvalidOperation":  # pragma: no cover
-                    LOGGER.error(
-                        "Data validation failed and cannot load to destination. RECORD: {}\n"
-                        "'multipleOf' validations that allows long precisions are not supported"
-                        " (i.e. with 15 digits or more). Try removing 'multipleOf' methods from JSON schema."
-                        .format(record_to_load))
-                    raise ex
+            # NOTE: Validate record
+            validators[stream].validate(float_to_decimal(record_to_load))
 
             if config.get('add_metadata_columns'):
-                record_to_load = add_metadata_values_to_record(o, {}, now.timestamp())
+                record_to_load = add_metadata_values_to_record(o, {}, now)
             else:
                 record_to_load = remove_metadata_values_from_record(o)
 
             file_data[stream]['file_data'].append(json.dumps(record_to_load) + '\n')
 
             # NOTE: write the lines into the temporary file when received data over 64Mb default memory buffer
-            if sys.getsizeof(file_data[stream]['file_data']) > config.get('memory_buffer', 64e6):
-                save_file(file_data[stream], open_func)
+            if sys.getsizeof(file_data[stream]['file_data']) > config.get('memory_buffer'):
+                save_records(file_data[stream], config.get('open_func'))
 
             state = None
         elif message_type == 'STATE':
@@ -230,6 +176,9 @@ def persist_lines(messages, config):
             else:
                 schemas[stream] = float_to_decimal(o['schema'])
 
+            # NOTE: prevent exception *** jsonschema.exceptions.UnknownType: Unknown type 'SCHEMA' for validator.
+            #       'type' is a key word for jsonschema validator which is different from `{'type': 'SCHEMA'}` as the message type.
+            schemas[stream].pop('type')
             validators[stream] = Draft4Validator(schemas[stream], format_checker=FormatChecker())
 
             if 'key_properties' not in o:
@@ -241,12 +190,11 @@ def persist_lines(messages, config):
             if stream not in file_data:
                 file_data[stream] = {
                     'target_key': get_target_key(
-                        o,
-                        naming_convention=naming_convention,
-                        timestamp=now_formatted,
-                        prefix=config.get('s3_key_prefix', ''),
-                        timezone=timezone),
-                    'file_name': temp_dir / naming_convention_default.format(stream=stream, timestamp=now_formatted),
+                        stream=stream,
+                        config=config,
+                        timestamp=now,
+                        prefix=config.get('s3_key_prefix', '')),
+                    'file_name': temp_dir / config['naming_convention_default'].format(stream=stream, timestamp=now),
                     'file_data': []}
 
         elif message_type == 'ACTIVATE_VERSION':
@@ -255,28 +203,112 @@ def persist_lines(messages, config):
             LOGGER.warning('Unknown message type "{}" in message "{}"'.format(o['type'], o))
 
     for _, file_info in file_data.items():
-        save_file(file_info, open_func)
+        save_records(file_info, config.get('open_func'))
 
     return state, file_data
 
 
+def config_file(config_path):
+    datetime_format = {
+        'timestamp_format': '%Y%m%dT%H%M%S',
+        'date_format': '%Y%m%d'
+    }
+
+    naming_convention_default = '{stream}-{timestamp}.json' \
+        .replace('{timestamp}', '{timestamp:' + datetime_format['timestamp_format'] + '}') \
+        .replace('{date}', '{date:' + datetime_format['date_format'] + '}')
+
+    config = {
+        'compression': 'none',
+        'naming_convention': naming_convention_default,
+        'memory_buffer': 64e6
+    }
+
+    with open(config_path) as input_file:
+        config.update(json.load(input_file))
+
+    missing_params = {'s3_bucket'} - set(config.keys())
+    if missing_params:
+        raise Exception(f'Config is missing required settings: {missing_params}')
+
+    unknown_params = set(config.keys()) - {
+        'add_metadata_columns',
+        'aws_access_key_id',
+        'aws_secret_access_key',
+        'aws_session_token',
+        'aws_endpoint_url',
+        'aws_profile',
+        's3_bucket',
+        's3_key_prefix',
+        'encryption_type',
+        'encryption_key',
+        'compression',
+        'naming_convention',
+        'timezone_offset',
+        'temp_dir',
+        'local',
+        'memory_buffer'
+    }
+
+    if unknown_params:
+        raise Exception(f'Config unknown settings: {unknown_params}')
+
+    config['naming_convention_default'] = naming_convention_default
+    config['naming_convention'] = config['naming_convention'] \
+        .replace('{timestamp}', '{timestamp:' + datetime_format['timestamp_format'] + '}') \
+        .replace('{date}', '{date:' + datetime_format['date_format'] + '}')
+
+    if f"{config.get('compression')}".lower() == 'gzip':
+        config['open_func'] = gzip.open
+        config['naming_convention_default'] = config['naming_convention_default'] + '.gz'
+        config['naming_convention'] = config['naming_convention'] + '.gz'
+
+    elif f"{config.get('compression')}".lower() == 'lzma':
+        config['open_func'] = lzma.open
+        config['naming_convention_default'] = config['naming_convention_default'] + '.xz'
+        config['naming_convention'] = config['naming_convention'] + '.xz'
+
+    elif f"{config.get('compression')}".lower() == 'none':
+        config['open_func'] = open
+
+    else:
+        raise NotImplementedError(
+            "Compression type '{}' is not supported. "
+            "Expected: 'none', 'gzip', or 'lzma'"
+            .format(f"{config.get('compression')}".lower()))
+
+    return config
+
+
+def upload_files(file_data, config):
+    if not config.get('local', False):
+        s3_client = s3.create_client(config)
+        for stream, file_info in file_data.items():
+            if file_info['file_name'].exists():
+                s3.upload_file(
+                    s3_client,
+                    str(file_info['file_name']),
+                    config.get('s3_bucket'),
+                    file_info['target_key'],
+                    encryption_type=config.get('encryption_type'),
+                    encryption_key=config.get('encryption_key'))
+                LOGGER.debug("{} file {} uploaded to {}".format(stream, file_info['target_key'], config.get('s3_bucket')))
+
+                # NOTE: Remove the local file(s)
+                file_info['file_name'].unlink()
+
+
 def main():
     parser = argparse.ArgumentParser()
     parser.add_argument('-c', '--config', help='Config file', required=True)
     args = parser.parse_args()
 
-    with open(args.config) as input_file:
-        config = json.load(input_file)
-
-    missing_params = {'s3_bucket'} - set(config.keys())
-    if missing_params:
-        raise Exception('Config is missing required keys: {}'.format(missing_params))
+    config = config_file(args.config)
 
     state, file_data = persist_lines(sys.stdin, config)
 
     # NOTE: Upload created files to S3
-    if not config.get('local', False):
-        upload_files(file_data, config)
+    upload_files(file_data, config)
 
     emit_state(state)
     LOGGER.debug('Exiting normally')
