@@ -10,15 +10,16 @@ LOGGER = get_logger()
 
 
 def retry_pattern():
-    return backoff.on_exception(backoff.expo,
-                                ClientError,
-                                max_tries=5,
-                                on_backoff=log_backoff_attempt,
-                                factor=10)
+    return backoff.on_exception(
+        backoff.expo,
+        ClientError,
+        max_tries=5,
+        on_backoff=log_backoff_attempt,
+        factor=10)
 
 
 def log_backoff_attempt(details):
-    LOGGER.info("Error detected communicating with Amazon, triggering backoff: %d try", details.get("tries"))  # pragma: no cover
+    LOGGER.info("Error detected communicating with Amazon, triggering backoff: %d try", details.get("tries"))
 
 
 @retry_pattern()
@@ -37,23 +38,20 @@ def create_client(config):
         aws_session = boto3.session.Session(
             aws_access_key_id=aws_access_key_id,
             aws_secret_access_key=aws_secret_access_key,
-            aws_session_token=aws_session_token
-        )
+            aws_session_token=aws_session_token)
     # AWS Profile based authentication
     else:
         aws_session = boto3.session.Session(profile_name=aws_profile)
 
     if aws_endpoint_url:
-        s3 = aws_session.client('s3', endpoint_url=aws_endpoint_url)
+        return aws_session.client('s3', endpoint_url=aws_endpoint_url)
     else:
-        s3 = aws_session.client('s3')
-
-    return s3
+        return aws_session.client('s3')
 
 
 # pylint: disable=too-many-arguments
 @retry_pattern()
-def upload_file(filename, s3_client, bucket, s3_key,
+def upload_file(s3_client, filename, bucket, s3_key,
                 encryption_type=None, encryption_key=None):
 
     if encryption_type is None or encryption_type.lower() == "none":
@@ -61,23 +59,17 @@ def upload_file(filename, s3_client, bucket, s3_key,
         encryption_desc = ""
         encryption_args = None
     elif encryption_type.lower() == "kms":
-        encryption_args = {"ServerSideEncryption": "aws:kms"}
         if encryption_key:
-            encryption_desc = (
-                " using KMS encryption key ID '{}'"
-                .format(encryption_key)
-            )
-            encryption_args["SSEKMSKeyId"] = encryption_key
+            encryption_desc = " using KMS encryption key ID '{}'".format(encryption_key)
+            encryption_args = {"ServerSideEncryption": "aws:kms", "SSEKMSKeyId": encryption_key}
         else:
             encryption_desc = " using default KMS encryption"
+            encryption_args = {"ServerSideEncryption": "aws:kms"}
     else:
         raise NotImplementedError(
             "Encryption type '{}' is not supported. "
             "Expected: 'none' or 'KMS'"
-            .format(encryption_type)
-        )
-    LOGGER.info(
-        "Uploading {} to bucket {} at {}{}"
-        .format(filename, bucket, s3_key, encryption_desc)
-    )
+            .format(encryption_type))
+
+    LOGGER.info("Uploading {} to bucket {} at {}{}".format(filename, bucket, s3_key, encryption_desc))
     s3_client.upload_file(filename, bucket, s3_key, ExtraArgs=encryption_args)
